## ----derivexp, eval=TRUE------------------------------------------------------
partialderiv <- D(expression(a * (x ^ b)),"b")
print(partialderiv)

## ----exprob1------------------------------------------------------------------
# Here we set up an example problem with data
# Define independent variables
t1 <- 1:20
t0 <- 0:19
t0a <- t0
t0a[1] <- 1e-7
y1 <- 4 * (t1^0.25)
n <- length(t1)
fuzz <- rnorm(n)
range <- max(y1)-min(y1)
y1p <- y1 + 0.1*range*fuzz
y1q <- y1 + 0.2*range*fuzz
y1r <- y1 + 0.5*range*fuzz
edta <- data.frame(t0=t0, t1=t1, t0a=t0a,y1=y1, y1p=y1p, y1q=y1q, y1r=y1r)

## ----extry1-------------------------------------------------------------------
start1 <- c(a=1, b=1)
try(nlsy1t1 <- nls(formula=y1~a*(t1^b), start=start1, data=edta))
try(nlsy1t1 <- nls(formula=y1~a*(t1^b), start=start1, data=edta, control=nls.control(maxiter=10000)))
library(nlsr)
nlsry1t1 <- nlxb(formula=y1~a*(t1^b), start=start1, data=edta)
nlsry1t1
library(minpack.lm)
nlsLMy1t1 <- nlsLM(formula=y1~a*(t1^b), start=start1, data=edta)
nlsLMy1t1



## ----extry2-------------------------------------------------------------------
try(nlsy1t0 <- nls(formula=y1~a*(t0^b), start=start1, data=edta))
nlsy1t0
library(nlsr)
try(nlsry1t0 <- nlxb(formula=y1~a*(t0^b), start=start1, data=edta))
##FAIL -- nlsry1t0
library(minpack.lm)
nlsLMy1t0 <- nlsLM(formula=y1~a*(t0^b), start=start1, data=edta)
nlsLMy1t0

try(nlsy1pt1 <- nls(formula=y1p~a*(t1^b), start=start1, data=edta))
nlsy1pt1
library(nlsr)
nlsry1pt1 <- nlxb(formula=y1p~a*(t1^b), start=start1, data=edta)
nlsry1pt1
library(minpack.lm)
nlsLMy1pt1 <- nlsLM(formula=y1p~a*(t1^b), start=start1, data=edta)
nlsLMy1pt1

try(nlsy1qt1 <- nls(formula=y1q~a*(t1^b), start=start1, data=edta))
nlsy1qt1
library(nlsr)
nlsry1qt1 <- nlxb(formula=y1q~a*(t1^b), start=start1, data=edta)
nlsry1qt1
library(minpack.lm)
nlsLMy1qt1 <- nlsLM(formula=y1q~a*(t1^b), start=start1, data=edta)
nlsLMy1qt1

try(nlsy1rt1 <- nls(formula=y1r~a*(t1^b), start=start1, data=edta))
nlsy1rt1
library(nlsr)
nlsry1rt1 <- nlxb(formula=y1r~a*(t1^b), start=start1, data=edta)
nlsry1rt1
library(minpack.lm)
nlsLMy1rt1 <- nlsLM(formula=y1r~a*(t1^b), start=start1, data=edta)
nlsLMy1rt1

## ----nlsstr-------------------------------------------------------------------
str(nlsy1pt1)

## ----nlsrstr------------------------------------------------------------------
str(nlsry1pt1)

## ----pred1--------------------------------------------------------------------
nudta <- colMeans(edta)
predict(nlsy1pt1, newdata=nudta)
predict(nlsLMy1pt1, newdata=nudta)
predict(nlsry1pt1, newdata=nudta)


## ----brownden-----------------------------------------------------------------
m <- 20
t <- seq(1, m) / 5
y <- rep(0,m)
library(nlsr)
library(minpack.lm)

bddata <- data.frame(t=t, y=y)
bdform <- y ~ ((x1 + t * x2 - exp(t))^2 + (x3 + x4 * sin(t) - cos(t))^2)
prm0 <- c(x1=25, x2=5, x3=-5, x4=-1)
fbd <-model2ssgrfun(bdform, prm0, bddata)
cat("initial sumsquares=",as.numeric(crossprod(fbd(prm0))),"\n")
## prmstar <- 
nlsrbd <- nlxb(bdform, start=prm0, data=bddata, trace=FALSE)
## summary(nlsrbd)
nlsrbd

nlsbd <- try(nls(bdform, start=prm0, data=bddata, trace=FALSE))
nlsbd

nlsbd10k <- nls(bdform, start=prm0, data=bddata, trace=FALSE, control=nls.control(maxiter=10000))
nlsbd10k

nlsLMbd <- nlsLM(bdform, start=prm0, data=bddata, trace=FALSE)
nlsLMbd

nlsLMbd10k <- nlsLM(bdform, start=prm0, data=bddata, trace=FALSE, control=nls.lm.control(maxiter=10000, maxfev=10000))
nlsLMbd10k

## ----browndenpred-------------------------------------------------------------
ndata <- data.frame(t=c(5,6), y=c(0,0))
predict(nlsLMbd, newdata=ndata)

# now nls
predict(nlsbd10k, newdata=ndata)

# now nlsr
predict(nlsrbd, newdata=ndata)



## ----brownden2----------------------------------------------------------------
bdf2 <-  (x1 + t * x2 - exp(t))^2 ~ - (x3 + x4 * sin(t) - cos(t))^2

nlsbd2 <- try(nls(bdf2, start=prm0, data=bddata, trace=FALSE))
nlsbd2

nlsLMbd2 <- try(nlsLM(bdf2, start=prm0, data=bddata, trace=FALSE, 
                      control=nls.lm.control(maxiter=10000, maxfev=10000)))
nlsLMbd2

nlsrbd2 <- nlxb(bdf2, start=prm0, data=bddata, trace=FALSE)
##summary(nlsrbd2)
nlsrbd2

## ----brownden2pred------------------------------------------------------------
ndata <- data.frame(t=c(5,6), y=c(0,0))
predict(nlsrbd2, newdata=ndata)

## ----bdcheck------------------------------------------------------------------
#' Brown and Dennis Function
#'
#' Test function 16 from the More', Garbow and Hillstrom paper.
#'
#' The objective function is the sum of \code{m} functions, each of \code{n}
#' parameters.
#'
#' \itemize{
#'   \item Dimensions: Number of parameters \code{n = 4}, number of summand
#'   functions \code{m >= n}.
#'   \item Minima: \code{f = 85822.2} if \code{m = 20}.
#' }
#'
#' @param m Number of summand functions in the objective function. Should be
#'   equal to or greater than 4.
#' @return A list containing:
#' \itemize{
#'   \item \code{fn} Objective function which calculates the value given input
#'   parameter vector.
#'   \item \code{gr} Gradient function which calculates the gradient vector
#'   given input parameter vector.
#'   \item \code{fg} A function which, given the parameter vector, calculates
#'   both the objective value and gradient, returning a list with members
#'   \code{fn} and \code{gr}, respectively.
#'   \item \code{x0} Standard starting point.
#' }
#' @references
#' More', J. J., Garbow, B. S., & Hillstrom, K. E. (1981).
#' Testing unconstrained optimization software.
#' \emph{ACM Transactions on Mathematical Software (TOMS)}, \emph{7}(1), 17-41.
#' \url{https://doi.org/10.1145/355934.355936}
#'
#' Brown, K. M., & Dennis, J. E. (1971).
#' \emph{New computational algorithms for minimizing a sum of squares of
#' nonlinear functions} (Report No. 71-6).
#' New Haven, CT: Department of Computer Science, Yale University.
#'
#' @examples
#' # Use 10 summand functions
#' fun <- brown_den(m = 10)
#' # Optimize using the standard starting point
#' x0 <- fun$x0
#' res_x0 <- stats::optim(par = x0, fn = fun$fn, gr = fun$gr, method =
#' "L-BFGS-B")
#' # Use your own starting point
#' res <- stats::optim(c(0.1, 0.2, 0.3, 0.4), fun$fn, fun$gr, method =
#' "L-BFGS-B")
#'
#' # Use 20 summand functions
#' fun20 <- brown_den(m = 20)
#' res <- stats::optim(fun20$x0, fun20$fn, fun20$gr, method = "L-BFGS-B")
#' @export
#`
brown_den <- function(m = 20) {
  list(
    fn = function(par) {
      x1 <- par[1]
      x2 <- par[2]
      x3 <- par[3]
      x4 <- par[4]

      ti <- (1:m) * 0.2
      l <- x1 + ti * x2 - exp(ti)
      r <- x3 + x4 * sin(ti) - cos(ti)
      f <- l * l + r * r
      sum(f * f)
    },
    gr = function(par) {
      x1 <- par[1]
      x2 <- par[2]
      x3 <- par[3]
      x4 <- par[4]

      ti <- (1:m) * 0.2
      sinti <- sin(ti)
      l <- x1 + ti * x2 - exp(ti)
      r <- x3 + x4 * sinti - cos(ti)
      f <- l * l + r * r
      lf4 <- 4 * l * f
      rf4 <- 4 * r * f
      c(
        sum(lf4),
        sum(lf4 * ti),
        sum(rf4),
        sum(rf4 * sinti)
      )
    },
    fg = function(par) {
      x1 <- par[1]
      x2 <- par[2]
      x3 <- par[3]
      x4 <- par[4]

      ti <- (1:m) * 0.2
      sinti <- sin(ti)
      l <- x1 + ti * x2 - exp(ti)
      r <- x3 + x4 * sinti - cos(ti)
      f <- l * l + r * r
      lf4 <- 4 * l * f
      rf4 <- 4 * r * f

      fsum <- sum(f * f)
      grad <- c(
        sum(lf4),
        sum(lf4 * ti),
        sum(rf4),
        sum(rf4 * sinti)
      )

      list(
        fn = fsum,
        gr = grad
      )
    },
    x0 = c(25, 5, -5, 1)
  )
}
mbd <- brown_den(m=20)
mbd
mbd$fg(mbd$x0)
bdsolnm <- optim(mbd$x0, mbd$fn, control=list(trace=0))
bdsolnm
bdsolbfgs <- optim(mbd$x0, mbd$fn, method="BFGS", control=list(trace=0))
bdsolbfgs

library(optimx)
methlist <- c("Nelder-Mead","BFGS","Rvmmin","L-BFGS-B","Rcgmin","ucminf")
solo <- opm(mbd$x0, mbd$fn, mbd$gr, method=methlist, control=list(trace=0))
summary(solo, order=value)


## ----smallres-----------------------------------------------------------------
x <- (1:m)
y <- 2.5 * (x ^ 0.33)
fmla <- y ~ a * (x^b)
edta <- data.frame(x=x, y=y)
library(nlsr)
library(minpack.lm)
enls <- try(nls(fmla, start=c(a=1, b=1), data=edta))
enlxb <- try(nlxb(fmla, start=c(a=1, b=1), data=edta))
enlxb
enlsLM <- try(nlsLM(fmla, start=c(a=1, b=1), data=edta))
enlsLM

